@@ -1,5 +1,5 @@
 package EntityModel;
-# ABSTRACT: Define and manage entities across languages
+# ABSTRACT: Cross-language event-driven ORM
 use EntityModel::Class {
 	_isa		=> [qw(EntityModel::Model)],
 	name		=> { type => 'string' },
@@ -9,7 +9,7 @@ use EntityModel::Class {
 	db		=> { type => 'EntityModel::DB' },
 };
 
-our $VERSION = '0.010';
+our $VERSION = '0.011';
 
 =head1 NAME
 
@@ -17,7 +17,7 @@ EntityModel - manage entity model definitions
 
 =head1 VERSION
 
-version 0.010
+version 0.011
 
 =head1 SYNOPSIS
 
@@ -40,15 +40,20 @@ version 0.010
  my $article = Entity::Article->create(
  	title => 'Test article',
 	content => 'Article content'
- );
- say "ID was " . $article->id;
-
- my ($match) = Entity::Article->find(
+ )->done(sub {
+  	my $article = shift;
+	say "ID was " . $article->id;
+ })->fail(sub {
+ 	die 'Failed to create new article';
+ });
+ Entity::Article->find(
  	title => 'Test article'
- );
- $match->title('Revised title');
- die "Instances of the same object should always be linked, consistent and up-to-date"
- 	unless $article->title eq $match->title;
+ )->first(sub {
+ 	my $match = shift;
+ 	$match->title('Revised title');
+ 	die "Instances of the same object should always be linked, consistent and up-to-date"
+ 		unless $article->title eq $match->title;
+ });
 
 =head1 DESCRIPTION
 
@@ -59,6 +64,9 @@ database tables, caching layer and the corresponding code (Perl/C++/JS) for acce
 A brief comparison and list of alternatives is in the L</MOTIVATION> and L</SEE ALSO> sections, please check there
 before investing any time into using this module.
 
+Eventually a full set of documentation will be added to L<http://entitymodel.com/documentation.html>
+but for now see the examples further down in this document.
+
 =head1 METHODS
 
 =cut
@@ -322,11 +330,13 @@ __END__
 
 A model contains metadata and zero or more entities.
 
-Each entity typically represents something that is able to be instantiated as an object, such as a row in a table. Since this module is
-heavily biased towards SQL-style applications, most of the entity definition is similar to SQL-92, with some additional features suitable
-for ORM-style access via other languages (Perl, JS, C++).
+Each entity typically represents something that is able to be instantiated as an object, such as a
+row in a table. Since this module is heavily biased towards SQL-style applications, most of the entity
+definition is similar to SQL-92, with some additional features suitable for ORM-style access via other
+languages (Perl, JS, C++).
 
-An entity definition primarily contains the following information - see L<EntityModel::Entity> for more details:
+An entity definition primarily contains the following information - see L<EntityModel::Entity> for more
+details:
 
 =over 4
 
@@ -398,8 +408,7 @@ and helper functions and methods. This also enables strict, warnings and Perl 5.
 Logging is handled through L<EntityModel::Log>, which imports functions such as logDebug.
 
 Arrays and hashes are typically wrapped using L<EntityModel::Array> and L<EntityModel::Hash>
-respectively, these provide similar functionality to L<autobox> at the cost of a slight
-performance hit.
+respectively, similar in concept to L<autobox>.
 
 For error handling, an L<EntityModel::Error> object is returned - this allows chained method
 calling without having to wrap in eval or check the result of each step when you don't care
@@ -540,16 +549,19 @@ Given the 'article' example from earlier, the Perl code is now:
  my $article = Entity::Article->create(
  	title => 'Test article',
 	content => 'Article content'
- );
- $article->on_create(sub {
+ )->done(sub {
  	my $a = shift;
 	say "ID was " . $a->id;
+ })->fail(sub {
+ 	die 'Failed to create the requested article :(';
  });
- Entity::Article->find(title => 'Test article')->on_result(sub {
+ Entity::Article->find(title => 'Test article')->each(sub {
  	my $match = shift;
 	$match->title('Revised title');
 	die "Instances of the same object should always be linked, consistent and up-to-date"
 		unless $article->title eq $match->title;
+ })->none(sub {
+	die 'Nothing found';
  });
 
 =head2 EXPORTING MODEL DEFINITIONS
@@ -572,7 +584,8 @@ Currently there is support for SQL and Perl model export, but not for Javascript
 
 =head3 AUDITING
 
-Audit tables are generated by default in the _audit schema, following the same naming convention as the audited tables with the addition of the following columns:
+Audit tables are generated by default in the _audit schema, following the same naming convention as the audited
+tables with the addition of the following columns:
 
 =over 4
 
@@ -624,43 +637,56 @@ The following classes provide features that are used throughout the code:
 
 =back
 
+See L<http://entitymodel.com/documentation.html> for more details and diagrams.
+
 =head1 MOTIVATION
 
-Some of the primary motivations for this distribution over any of the existing approaches (see next section for some alternatives):
+Some of the primary motivations for this distribution over any of the existing approaches (see next
+section for some alternatives):
 
 =over 4
 
-=item * B<Support for languages other than Perl> - most projects end up using at least one other language, e.g. web-based projects typically have some
-Javascript on the frontend talking to the Perl backend.
+=item * B<Event-based operation> - backend storage requests run 'asynchronously' where possible,
+allowing the application to continue with other tasks while waiting for database queries to complete.
 
-=item * B<Single configuration file> - I like to be able to export, backup and diff the entity layout and having this in a single file as JSON or XML is
-more convenient for me than using multiple Perl packages, this also allows the configuration to be used by non-Perl code.
+=item * B<Support for languages other than Perl> - most projects end up using at least one other
+language, e.g. web-based projects typically have some Javascript on the frontend talking to the
+Perl backend.
 
-=item * B<Backend abstraction> - the conceptual model generally doesn't need to be tied to a particular backend, for example the Perl side of things could
-either talk directly to a database or use a webservice.
+=item * B<Configurable with a single file> - I like to be able to export, backup and diff the entity
+layout and having this in a single file as JSON or XML is more convenient for me than using multiple
+Perl packages, this also allows the configuration to be used by non-Perl code.
 
-=item * B<Easy editing and visualisation> - I like to have the option of using other tools such as diagram editors to add/modify the entity layout, rather
-than having to create or edit Perl files. This helps to separate actual code changes from configuration / layout issues; the L<EntityModel> distribution
+=item * B<Backend abstraction> - the conceptual model generally doesn't need to be tied to a particular
+backend, for example the Perl side of things could either talk directly to a database or use a webservice.
+
+=item * B<Easy editing and visualisation> - I like to have the option of using other tools such as
+diagram editors to add/modify the entity layout, rather than having to create or edit Perl files. This
+helps to separate actual code changes from configuration / layout issues; the L<EntityModel> distribution
 can be installed once and for many common applications no further custom code should be required.
 
-=item * B<Flexibility> - although too much interdependence is generally a bad thing, being able to include other concepts in the configuration file has been
-useful for tasks such as building websites with common features.
+=item * B<Flexibility> - although too much interdependence is generally a bad thing, being able to include
+other concepts in the configuration file has been useful for tasks such as building websites with common
+features.
 
 =back
 
-Clearly most, if not all, alternative systems could be adapted to support the above requirements.
+Clearly none of these features are necessarily unique to L<EntityModel>, and many of the alternative
+systems described in the next section could be adapted to support the above requirements.
 
 =head1 SEE ALSO
 
-There are plenty of other, better ORM implementations available on CPAN:
+There are plenty of other, better ORM implementations available on CPAN - these are the ones I've found so far:
 
 =over 4
 
-=item * L<DBIx::Class> - appears to be the most highly regarded and actively developed one out there, and the available features, code quality and general
-stability are far in advance of this module so unless you need the L<EntityModel> multi-language export features I would encourage people to look here first.
+=item * L<DBIx::Class> - appears to be the most highly regarded and actively developed one out there, and
+the available features, code quality and general stability are far in advance of this module. Unless you
+need the L<EntityModel> multi-language or asynchronous support features I would strongly encourage looking
+at L<DBIx::Class> first.
 
-=item * L<Rose::DB::Object> - written for speed, appears to cover most of the usual requirements, personally found the API less intuitive than other options
-but it appears to be widely deployed.
+=item * L<Rose::DB::Object> - written for speed, appears to cover most of the usual requirements, personally
+found the API less intuitive than other options but it appears to be widely deployed.
 
 =item * L<Fey::ORM> - newer than the other options, also appears to be reasonably flexible.
 
@@ -668,13 +694,25 @@ but it appears to be widely deployed.
 
 =item * L<Alzabo> - another ORM which includes features such as GUI schema editing and SQL diff
 
-=item * L<Class::DBI> - generally considered to be superceded by L<DBIx::Class>, which provides a compatibility layer for existing applications.
+=item * L<Class::DBI> - generally considered to be superceded by L<DBIx::Class>, which provides a compatibility
+layer for existing applications.
+
+=item * L<ORMesque> - lightweight class-based ORM using L<SQL::Abstract>
+
+=item * L<Oryx> - Object persistence framework, meta-model based with support for both DBM and regular RDBMS
+backends, uses tied hashes and arrays.
+
+=item * L<Tangram> - An object persistence layer.
+
+=item * L<KiokuDB> - described as an "Object Graph storage engine" rather than an ORM
 
 =back
 
 Distributions which provide class structure and wrappers around the Perl OO mechanism are likewise covered by
 several other CPAN modules, with the clear winner here in the form of L<Moose> and derivatives.
 
+Eventually I'll try to put up a better set of comparisons on L<http://entitymodel.com>.
+
 =head1 AUTHOR
 
 Tom Molesworth <cpan@entitymodel.com>
